from kivy.lang import Builder

# fmt: off
Builder.load_string("""
#:import rgba kivy.utils.get_color_from_hex

#:set slate_50 rgba("#f8fafc")
#:set slate_100 rgba("#f1f5f9")
#:set slate_200 rgba("#e2e8f0")
#:set slate_300 rgba("#cbd5e1")
#:set slate_400 rgba("#94a3b8")
#:set slate_500 rgba("#64748b")
#:set slate_600 rgba("#475569")
#:set slate_700 rgba("#334155")
#:set slate_800 rgba("#1e293b")
#:set slate_900 rgba("#0f172a")

#:set slate slate_500

#:set gray_50 rgba("#f9fafb")
#:set gray_100 rgba("#f3f4f6")
#:set gray_200 rgba("#e5e7eb")
#:set gray_300 rgba("#d1d5db")
#:set gray_400 rgba("#9ca3af")
#:set gray_500 rgba("#6b7280")
#:set gray_600 rgba("#4b5563")
#:set gray_700 rgba("#374151")
#:set gray_800 rgba("#1f2937")
#:set gray_900 rgba("#111827")

#:set gray gray_500

#:set zinc_50 rgba("#fafafa")
#:set zinc_100 rgba("#f4f4f5")
#:set zinc_200 rgba("#e4e4e7")
#:set zinc_300 rgba("#d4d4d8")
#:set zinc_400 rgba("#a1a1aa")
#:set zinc_500 rgba("#71717a")
#:set zinc_600 rgba("#52525b")
#:set zinc_700 rgba("#3f3f46")
#:set zinc_800 rgba("#27272a")
#:set zinc_900 rgba("#18181b")

#:set zinc zinc_500

#:set neutral_50 rgba("#fafafa")
#:set neutral_100 rgba("#f5f5f5")
#:set neutral_200 rgba("#e5e5e5")
#:set neutral_300 rgba("#d4d4d4")
#:set neutral_400 rgba("#a3a3a3")
#:set neutral_500 rgba("#737373")
#:set neutral_600 rgba("#525252")
#:set neutral_700 rgba("#404040")
#:set neutral_800 rgba("#262626")
#:set neutral_900 rgba("#171717")

#:set neutral neutral_500

#:set stone_50 rgba("#fafaf9")
#:set stone_100 rgba("#f5f5f4")
#:set stone_200 rgba("#e7e5e4")
#:set stone_300 rgba("#d6d3d1")
#:set stone_400 rgba("#a8a29e")
#:set stone_500 rgba("#78716c")
#:set stone_600 rgba("#57534e")
#:set stone_700 rgba("#44403c")
#:set stone_800 rgba("#292524")
#:set stone_900 rgba("#1c1917")

#:set stone stone_500

#:set red_50 rgba("#fef2f2")
#:set red_100 rgba("#fee2e2")
#:set red_200 rgba("#fecaca")
#:set red_300 rgba("#fca5a5")
#:set red_400 rgba("#f87171")
#:set red_500 rgba("#ef4444")
#:set red_600 rgba("#dc2626")
#:set red_700 rgba("#b91c1c")
#:set red_800 rgba("#991b1b")
#:set red_900 rgba("#7f1d1d")

#:set red red_500

#:set orange_50 rgba("#fff7ed")
#:set orange_100 rgba("#ffedd5")
#:set orange_200 rgba("#fed7aa")
#:set orange_300 rgba("#fdba74")
#:set orange_400 rgba("#fb923c")
#:set orange_500 rgba("#f97316")
#:set orange_600 rgba("#ea580c")
#:set orange_700 rgba("#c2410c")
#:set orange_800 rgba("#9a3412")
#:set orange_900 rgba("#7c2d12")

#:set orange orange_500

#:set amber_50 rgba("#fffbeb")
#:set amber_100 rgba("#fef3c7")
#:set amber_200 rgba("#fde68a")
#:set amber_300 rgba("#fcd34d")
#:set amber_400 rgba("#fbbf24")
#:set amber_500 rgba("#f59e0b")
#:set amber_600 rgba("#d97706")
#:set amber_700 rgba("#b45309")
#:set amber_800 rgba("#92400e")
#:set amber_900 rgba("#78350f")

#:set amber amber_500

#:set yellow_50 rgba("#fefce8")
#:set yellow_100 rgba("#fef9c3")
#:set yellow_200 rgba("#fef08a")
#:set yellow_300 rgba("#fde047")
#:set yellow_400 rgba("#facc15")
#:set yellow_500 rgba("#eab308")
#:set yellow_600 rgba("#ca8a04")
#:set yellow_700 rgba("#a16207")
#:set yellow_800 rgba("#854d0e")
#:set yellow_900 rgba("#713f12")

#:set yellow yellow_500

#:set lime_50 rgba("#f7fee7")
#:set lime_100 rgba("#ecfccb")
#:set lime_200 rgba("#d9f99d")
#:set lime_300 rgba("#bef264")
#:set lime_400 rgba("#a3e635")
#:set lime_500 rgba("#84cc16")
#:set lime_600 rgba("#65a30d")
#:set lime_700 rgba("#4d7c0f")
#:set lime_800 rgba("#3f6212")
#:set lime_900 rgba("#365314")

#:set lime lime_500

#:set green_50 rgba("#f0fdf4")
#:set green_100 rgba("#dcfce7")
#:set green_200 rgba("#bbf7d0")
#:set green_300 rgba("#86efac")
#:set green_400 rgba("#4ade80")
#:set green_500 rgba("#22c55e")
#:set green_600 rgba("#16a34a")
#:set green_700 rgba("#15803d")
#:set green_800 rgba("#166534")
#:set green_900 rgba("#14532d")

#:set green green_500

#:set emerald_50 rgba("#ecfdf5")
#:set emerald_100 rgba("#d1fae5")
#:set emerald_200 rgba("#a7f3d0")
#:set emerald_300 rgba("#6ee7b7")
#:set emerald_400 rgba("#34d399")
#:set emerald_500 rgba("#10b981")
#:set emerald_600 rgba("#059669")
#:set emerald_700 rgba("#047857")
#:set emerald_800 rgba("#065f46")
#:set emerald_900 rgba("#064e3b")

#:set emerald emerald_500

#:set teal_50 rgba("#f0fdfa")
#:set teal_100 rgba("#ccfbf1")
#:set teal_200 rgba("#99f6e4")
#:set teal_300 rgba("#5eead4")
#:set teal_400 rgba("#2dd4bf")
#:set teal_500 rgba("#14b8a6")
#:set teal_600 rgba("#0d9488")
#:set teal_700 rgba("#0f766e")
#:set teal_800 rgba("#115e59")
#:set teal_900 rgba("#134e4a")

#:set teal teal_500

#:set cyan_50 rgba("#ecfeff")
#:set cyan_100 rgba("#cffafe")
#:set cyan_200 rgba("#a5f3fc")
#:set cyan_300 rgba("#67e8f9")
#:set cyan_400 rgba("#22d3ee")
#:set cyan_500 rgba("#06b6d4")
#:set cyan_600 rgba("#0891b2")
#:set cyan_700 rgba("#0e7490")
#:set cyan_800 rgba("#155e75")
#:set cyan_900 rgba("#164e63")

#:set cyan cyan_500

#:set sky_50 rgba("#f0f9ff")
#:set sky_100 rgba("#e0f2fe")
#:set sky_200 rgba("#bae6fd")
#:set sky_300 rgba("#7dd3fc")
#:set sky_400 rgba("#38bdf8")
#:set sky_500 rgba("#0ea5e9")
#:set sky_600 rgba("#0284c7")
#:set sky_700 rgba("#0369a1")
#:set sky_800 rgba("#075985")
#:set sky_900 rgba("#0c4a6e")

#:set sky sky_500

#:set blue_50 rgba("#eff6ff")
#:set blue_100 rgba("#dbeafe")
#:set blue_200 rgba("#bfdbfe")
#:set blue_300 rgba("#93c5fd")
#:set blue_400 rgba("#60a5fa")
#:set blue_500 rgba("#3b82f6")
#:set blue_600 rgba("#2563eb")
#:set blue_700 rgba("#1d4ed8")
#:set blue_800 rgba("#1e40af")
#:set blue_900 rgba("#1e3a8a")

#:set blue blue_500

#:set indigo_50 rgba("#eef2ff")
#:set indigo_100 rgba("#e0e7ff")
#:set indigo_200 rgba("#c7d2fe")
#:set indigo_300 rgba("#a5b4fc")
#:set indigo_400 rgba("#818cf8")
#:set indigo_500 rgba("#6366f1")
#:set indigo_600 rgba("#4f46e5")
#:set indigo_700 rgba("#4338ca")
#:set indigo_800 rgba("#3730a3")
#:set indigo_900 rgba("#312e81")

#:set indigo indigo_500

#:set violet_50 rgba("#f5f3ff")
#:set violet_100 rgba("#ede9fe")
#:set violet_200 rgba("#ddd6fe")
#:set violet_300 rgba("#c4b5fd")
#:set violet_400 rgba("#a78bfa")
#:set violet_500 rgba("#8b5cf6")
#:set violet_600 rgba("#7c3aed")
#:set violet_700 rgba("#6d28d9")
#:set violet_800 rgba("#5b21b6")
#:set violet_900 rgba("#4c1d95")

#:set violet violet_500

#:set purple_50 rgba("#faf5ff")
#:set purple_100 rgba("#f3e8ff")
#:set purple_200 rgba("#e9d5ff")
#:set purple_300 rgba("#d8b4fe")
#:set purple_400 rgba("#c084fc")
#:set purple_500 rgba("#a855f7")
#:set purple_600 rgba("#9333ea")
#:set purple_700 rgba("#7e22ce")
#:set purple_800 rgba("#6b21a8")
#:set purple_900 rgba("#581c87")

#:set purple purple_500

#:set fuchsia_50 rgba("#fdf4ff")
#:set fuchsia_100 rgba("#fae8ff")
#:set fuchsia_200 rgba("#f5d0fe")
#:set fuchsia_300 rgba("#f0abfc")
#:set fuchsia_400 rgba("#e879f9")
#:set fuchsia_500 rgba("#d946ef")
#:set fuchsia_600 rgba("#c026d3")
#:set fuchsia_700 rgba("#a21caf")
#:set fuchsia_800 rgba("#86198f")
#:set fuchsia_900 rgba("#701a75")

#:set fuchsia fuchsia_500

#:set pink_50 rgba("#fdf2f8")
#:set pink_100 rgba("#fce7f3")
#:set pink_200 rgba("#fbcfe8")
#:set pink_300 rgba("#f9a8d4")
#:set pink_400 rgba("#f472b6")
#:set pink_500 rgba("#ec4899")
#:set pink_600 rgba("#db2777")
#:set pink_700 rgba("#be185d")
#:set pink_800 rgba("#9d174d")
#:set pink_900 rgba("#831843")

#:set pink pink_500

#:set rose_50 rgba("#fff1f2")
#:set rose_100 rgba("#ffe4e6")
#:set rose_200 rgba("#fecdd3")
#:set rose_300 rgba("#fda4af")
#:set rose_400 rgba("#fb7185")
#:set rose_500 rgba("#f43f5e")
#:set rose_600 rgba("#e11d48")
#:set rose_700 rgba("#be123c")
#:set rose_800 rgba("#9f1239")
#:set rose_900 rgba("#881337")

#:set rose rose_500

#:set white rgba("#ffffff")
#:set black rgba("#000000")
""")
# fmt: on



import os

from kivy.clock import Clock
from kivy.core.clipboard import Clipboard
from kivy.core.window import Window
from kivy.lang import Builder, global_idmap
from kivy.metrics import dp
from kivy.properties import (
    ColorProperty,
    ListProperty,
    NumericProperty,
    OptionProperty,
    StringProperty,
)
from kivy.uix.behaviors import ButtonBehavior
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.label import Label
from kivy.uix.relativelayout import RelativeLayout

from .icon_definitions import icon_unicodes

__file__ = os.path.abspath(__file__)
font_path = os.path.join(
    os.path.dirname(__file__), "fonts", "materialdesignicons-webfont.ttf"
)

global_idmap["unicode"] = icon_unicodes
global_idmap["icon_font"] = font_path


class Icon(Label):
    icon = StringProperty("android")
    font_name = StringProperty(font_path)
    icon_color = ColorProperty([0, 0, 0, 0.1])
    icon_size = NumericProperty(dp(21))
    bg_color = ColorProperty([0, 0, 0, 0])
    mode = OptionProperty(
        "default", options=["default", "stretch-background", "stretch-icon"]
    )


class IconViewer(RelativeLayout):
    icons = ListProperty()
    cache_icons = ListProperty()
    selected_icon = StringProperty()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.set_list_md_icons()
        self.cache_icons = self.icons

    def set_list_md_icons(self, text="", search=False):
        """
        Set list of icons to display
        """
        if search:
            self.icons = [
                {"icon": name_icon, "viewer": self}
                for name_icon in icon_unicodes.keys()
                if text in name_icon
            ]
        else:
            self.icons = [
                {"icon": name_icon, "viewer": self}
                for name_icon in icon_unicodes.keys()
            ]


class IconViewerItem(ButtonBehavior, BoxLayout):
    bg_color = ListProperty([0, 0, 0, 0.2])

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Window.bind(mouse_pos=self.check_collision)

    def check_collision(self, _, pos):
        """
        Checks if mouse is over icon,
        updates icon background color
        and tipbox position/opacity.
        """
        if self.collide_point(*self.to_widget(*pos)):
            self.bg_color = [0, 0, 0, 0.4]

            self.viewer.tipbox.pos = (
                self.to_window(*self.pos)[0] - self.width / 2,
                self.to_window(*self.pos)[1] + self.height,
            )

            def update_opacity(*args):
                self.viewer.tipbox.opacity = 1

            if self.viewer.selected_icon == self.icon:
                Clock.schedule_once(update_opacity)

            self.viewer.selected_icon = self.icon
        else:
            self.bg_color = [0, 0, 0, 0.2]
            self.viewer.tipbox.opacity = 0

    def on_release(self):
        """
        Copy icon name to clipboard
        and update tipbox text
        """
        Clipboard.copy(self.icon)
        self.viewer.tipbox.children[0].text = "Copied successfully!"


# fmt: off
Builder.load_string("""
<Icon>:
    text: u"{}".format(unicode[root.icon]) if root.icon in unicode else "blank"
    size_hint: (None, None) if root.mode == "default" else (1, 1)
    size: (root.icon_size, root.icon_size) if root.mode == "default" else (self.texture_size[0], self.texture_size[1]) # this is the background size
    font_size: root.icon_size if root.mode in ["default", "stretch-background"] else (root.size[0] if root.size[0] < root.size[1] else root.size[1]) # this is the icon size
    pos_hint: {"center_x": .5, "center_y": .5}
    color: root.icon_color
    
    canvas.before:
        Color:
            rgba: root.bg_color or (0, 0, 0, 0)
        Rectangle:
            pos: self.pos
            size: self.size

<IconViewer>:
    available_space: self.width - dp(30)
    number_of_spacings: int((root.available_space - 2*dp(20)) / dp(150)) - 1
    cols: int((root.available_space - 2*dp(20) - root.number_of_spacings*dp(10)) / dp(150))
    left_padding: (root.available_space - root.cols*dp(150) - (root.cols-1)*dp(10)) / 2

    rv_grid: rv_grid.__self__
    tipbox: tipbox.__self__

    BoxLayout:
        orientation: "vertical"
        padding: dp(15), 0
        spacing: dp(5)
        
        TextInput:
            id: search
            size_hint_y: None
            height: dp(40)
            hint_text: "Search"
            multiline: False
            on_text: root.set_list_md_icons(self.text, True)

        RecycleView:
            viewclass: "IconViewerItem"
            data: root.icons
            pos_hint: {"center_x": .5}

            bar_width: dp(20)
            bar_color: blue
            scroll_type: ["bars", "content"]
            canvas.before:
                Color:
                    rgba: white
                Rectangle:
                    pos: self.pos
                    size: self.size
            
            RecycleGridLayout:
                id: rv_grid
                cols: root.cols
                padding: root.left_padding, 0, dp(20), 0
                spacing: dp(10)
                default_size: dp(150), dp(100)
                default_size_hint: None, None
                size_hint_y: None
                height: self.minimum_height

    FloatLayout:
        BoxLayout:
            id: tipbox
            size_hint: None, None
            size: self.minimum_size
            padding: dp(14)
            opacity: 0

            canvas.before:
                Color:
                    rgba: white
                RoundedRectangle:
                    pos: self.pos
                    size: self.size
                    radius: [dp(5), dp(5), 0, 0]
            Label:
                size_hint: None, None
                size: self.texture_size
                text: "Click to copy icon name: \\n[b]{}".format(root.selected_icon)
                color: blue_400
                halign: "center"
                markup: True
                font_size: dp(21)
            

<IconViewerItem>:
    orientation: "vertical"
    icon: 'android'

    canvas.before:
        Color:
            rgba: gray_300
        Line:
            rectangle: self.x, self.y, self.width, self.height
            width: dp(1)

    canvas.after:
        Color:
            rgba: root.bg_color
        Rectangle:
            pos: self.pos
            size: self.size

    Icon:
        mode: "stretch-icon"
        icon: root.icon
        icon_color: rgba("#5F6368")
        bg_color: slate_200
    Label:
        text: root.icon
        font_size: dp(16)
        size_hint_y: None
        height: dp(20)
        color: black
        shorten: True
        shorten_from: "right"
        text_size: dp(135), None
        halign: "center"
""")



from kivy.animation import Animation
from kivy.factory import Factory
from kivy.lang import Builder, global_idmap
from kivy.metrics import dp
from kivy.properties import (
    AliasProperty,
    BooleanProperty,
    ColorProperty,
    ListProperty,
    NumericProperty,
    ObjectProperty,
    OptionProperty,
    StringProperty,
)
from kivy.uix.behaviors import ButtonBehavior
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.dropdown import DropDown
from kivy.uix.label import Label

from .color_definitions import *


class DropDownItem(ButtonBehavior, Label):
    color = ColorProperty([0, 0, 0, 1])
    text = StringProperty()
    bg_color = ListProperty([1, 1, 1, 1])

    def get_current_bg_color(self):
        if self.state == "normal":
            return self.bg_color
        else:
            return [
                self.bg_color[0] * 0.8,
                self.bg_color[1] * 0.8,
                self.bg_color[2] * 0.8,
                1,
            ]

    current_bg_color = AliasProperty(get_current_bg_color, bind=["state", "bg_color"])


# fmt: off
Builder.load_string("""
<DropDownItem>:
    size_hint_y: None
    height: dp(50)

    canvas.before:
        Color:
            rgba: root.current_bg_color or white
        Rectangle:
            size: self.size
            pos: self.pos

        Color:
            rgba: black
        Line:
            width: 1
            points: self.x, self.y+self.height, self.x+self.width, self.y+self.height

        Color:
            rgba: black
        Line:
            width: 1
            points: self.x, self.y, self.x, self.y+self.height

        Color:
            rgba: black
        Line:
            width: 1
            points: self.x+self.width, self.y, self.x+self.width, self.y+self.height
""")
# fmt: off


class Container(DropDown):
    container_position = OptionProperty(
        "auto", options=["auto", "left-top", "top", "right-top", "left", "center", "right", "left-bottom", "bottom", "right-bottom"]
    )
    bg_color = ColorProperty([1,1,1,1])

    def __init__(self, max_height=dp(200), **kwargs):
        super().__init__(**kwargs)
        self.max_height = max_height

    def _reposition(self, *largs):
        # calculate the coordinate of the attached widget in the window
        # coordinate system
        win = self._win
        if not win:
            return
        widget = self.attach_to
        if not widget or not widget.get_parent_window():
            return
        wx, wy = widget.to_window(*widget.pos)
        wright, wtop = widget.to_window(widget.right, widget.top)

        if self.auto_width:
            self.width = wright - wx
        else:
            self.width = widget.container_width
        
        if self.container_position == 'auto':
            # ensure the dropdown list doesn't get out on the X axis, with a
            # preference to 0 in case the list is too wide.
            x = wx
            if x + self.width > win.width:
                x = win.width - self.width
            if x < 0:
                x = 0
            self.x = x

            # determine if we display the dropdown upper or lower to the widget
            if self.max_height is not None:
                height = min(self.max_height, self.container.minimum_height)
            else:
                height = self.container.minimum_height

            h_bottom = wy - height
            h_top = win.height - (wtop + height)
            if h_bottom > 0:
                self.top = wy
                self.height = height
            elif h_top > 0:
                self.y = wtop
                self.height = height
            else:
                # none of both top/bottom have enough place to display the
                # widget at the current size. Take the best side, and fit to
                # it.
                if h_top < h_bottom:
                    self.top = self.height = wy
                else:
                    self.y = wtop
                    self.height = win.height - wtop

            return
        
        # self.height = widget.container_height

        if self.container_position == "left-top":
            self.x = wx - self.width/2
            self.y = wtop

        elif self.container_position == "top":
            self.x = wx - (self.width - widget.width)/2
            self.y = wtop

        elif self.container_position == "right-top":
            self.x = wx + widget.width / 2
            self.y = wtop

        elif self.container_position == "left":
            self.x = wx - self.width
            self.y = wy - self.height/2 + widget.height/2

        elif self.container_position == "center":
            self.x = wx - (self.width - widget.width)/2
            self.y = wy - self.height/2 + widget.height/2

        elif self.container_position == "right":
            self.x = wx + widget.width
            self.y = wy - self.height/2 + widget.height/2

        elif self.container_position == "left-bottom":
            self.x = wx - self.width/2
            self.y = wy - self.height

        elif self.container_position == "bottom":
            self.x = wx - (self.width - widget.width)/2
            self.y = wy - self.height

        elif self.container_position == "right-bottom":
            self.x = wx + widget.width/2
            self.y = wy - self.height


class CDropDown(ButtonBehavior, BoxLayout):
    values = ListProperty()
    text_autoupdate = BooleanProperty(False)
    dropdown_cls = ObjectProperty(Container)
    is_open = BooleanProperty(False)

    container_height = NumericProperty(dp(200))
    container_width = NumericProperty(0)
    container_position = OptionProperty(
        "auto", options=["auto", "left-top", "top", "right-top", "left", "center", "right", "left-bottom", "bottom", "right-bottom"]
    )
    container_bg_color = ColorProperty()

    text = StringProperty()
    font_size = NumericProperty(dp(18))
    font_color = ColorProperty([0,0,0,1])
    bold = BooleanProperty(False)
    
    icon = StringProperty("chevron-down")
    icon_color = ColorProperty(global_idmap["slate_500"])
    icon_size = NumericProperty(dp(21))

    border_color = ColorProperty(global_idmap["slate_400"])
    border_width = NumericProperty(1)

    bg_color = ListProperty([1, 1, 1, 1])
    radius = ListProperty([0, 0, 0, 0])

    def __init__(self, **kwargs):
        self._dropdown = None
        super().__init__(**kwargs)
        fbind = self.fbind
        build_dropdown = self._build_dropdown
        fbind("on_release", self._toggle_dropdown)
        fbind("dropdown_cls", build_dropdown)
        fbind("values", self._update_dropdown)
        fbind("text_autoupdate", self._update_dropdown)
        
        build_dropdown()

    def on_container_height(self, *args):
        self._dropdown.max_height = self.container_height

    def on_container_position(self, *args):
        self._dropdown.container_position = self.container_position
        self._dropdown._reposition()

    def on_container_bg_color(self, *args):
        self._dropdown.bg_color = self.container_bg_color


    def _build_dropdown(self, *largs):
        if self._dropdown:
            self._dropdown.unbind(on_select=self._on_dropdown_select)
            self._dropdown.unbind(on_dismiss=self._close_dropdown)
            self._dropdown.dismiss()
            self._dropdown = None
        cls = self.dropdown_cls
        if isinstance(cls, str):
            cls = Factory.get(cls)

        if not self.container_width:
            self._dropdown = cls(auto_width=True)
        else:
            self._dropdown = cls()


        self._dropdown.bind(on_select=self._on_dropdown_select)
        self._dropdown.bind(on_dismiss=self._close_dropdown)
        self._update_dropdown()

    def _update_dropdown(self, *largs):
        """
        Update the dropdown with the values. Each value is a dict with the
        following keys:
            - text: text displayed in the dropdown
            - viewclass: class used for displaying the text
            - anything else: added as an attribute to the viewclass

        """
        dp = self._dropdown
        values: dict[str:str] = self.values
        text_autoupdate = self.text_autoupdate
        dp.clear_widgets()
        for value in values: 
            classname = value.get("viewclass")

            if not classname:
                classname = "DropDownItem"
            
            copy_value = value.copy()
            copy_value.pop("viewclass", None)
            cls = Factory.get(classname)
            item = cls(**copy_value)
            item.size_hint_y = None

            height = copy_value.get("height")
            if height or hasattr(cls, "height"):
                item.height = height or item.height
            else:
                item.height = self.height
            item.bind(on_release=lambda option: dp.select(option.text))
            dp.add_widget(item)
        if text_autoupdate:
            if values:
                if not self.text or self.text not in values:
                    self.text = values[0]
            else:
                self.text = ""

    def _toggle_dropdown(self, *largs):
        if self.values:
            self.is_open = not self.is_open

    def _close_dropdown(self, *largs):
        self.is_open = False

    def _on_dropdown_select(self, instance: object, text: str, *largs):
        """
        Callback called when the dropdown select an option.
        e.g.: data = {'text': 'option1', 'viewclass': 'MyOption'}
        """
        self.text = text
        self.is_open = False

    def on_is_open(self, instance, value):
        if value:
            height = self.container_height
            self._dropdown.open(self)
            self._dropdown.height = 0
            Animation(height=height, d=0.2, t="out_cubic").start(self._dropdown)

        else:
            if self._dropdown.attach_to:
                self._dropdown.dismiss()

# fmt: off
Builder.load_string("""
<CDropDown>:
    padding: dp(10), 0
    size_hint: None, None
    height: dp(50)
    width: self.minimum_width
    
    canvas.before:
        Color:
            rgba: root.bg_color if self.state == "normal" else [root.bg_color[0]*0.9, root.bg_color[1]*0.9, root.bg_color[2]*0.9, root.bg_color[3]]
        RoundedRectangle:
            size: self.size
            pos: self.pos
            radius: root.radius
        Color:
            rgba: root.border_color
        SmoothLine:
            width: root.border_width
            rounded_rectangle: (self.x, self.y, self.width, self.height, root.radius[0]) if root.radius[0] else (self.x, self.y, self.width, self.height, 1)

    Label:
        markup: True
        text: root.text if not root.bold else f'[b]{root.text}[/b]'
        color: root.font_color
        font_size: root.font_size
        size_hint_x: None
        width: self.texture_size[0] + dp(10)
    Icon:
        icon: root.icon
        icon_color: root.icon_color
        icon_size: root.icon_size

<Container>
    canvas.before:
        Color:
            rgba: root.bg_color
        Rectangle:
            pos: self.pos
            size: self.size
""")



from kivy.lang import Builder
from kivy.metrics import dp, sp
from kivy.properties import (
    AliasProperty,
    ColorProperty,
    ListProperty,
    NumericProperty,
    StringProperty,
)
from kivy.uix.behaviors import ButtonBehavior
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.label import Label
from kivy.uix.widget import Widget


class CButton(ButtonBehavior, BoxLayout):
    """
    Custom button with icon and text
    """

    text = StringProperty()
    font_size = NumericProperty(sp(18))
    icon = StringProperty()
    bg_color = ColorProperty([1, 1, 1, 1])
    font_color = ColorProperty([0, 0, 0, 1])
    icon_color = ColorProperty([0, 0, 0, 1])
    radius = ListProperty([dp(5)])
    icon_size = NumericProperty(dp(24))
    border_color = ColorProperty([1, 1, 1, 1])
    border_width = NumericProperty(1)
    # icon_position = OptionProperty("left", options=["left", "right"])
    # text_pos = OptionProperty("left", options=["left", "center"])

    def get_current_color(self):
        if self.state == "normal":
            return self.bg_color
        else:
            return [
                self.bg_color[0] * 0.8,
                self.bg_color[1] * 0.8,
                self.bg_color[2] * 0.8,
                self.bg_color[3] * 1,
            ]

    current_color = AliasProperty(
        get_current_color,
        bind=["state", "bg_color"],
    )

    def get_current_mode(self):
        if self.text and self.icon:
            return "both"
        elif self.text:
            return "text"
        elif self.icon:
            return "icon"

    current_mode = AliasProperty(
        get_current_mode,
        bind=["text", "icon"],
    )

    def on_current_mode(self, *args):
        if len(self.children) > 2:
            children_to_remove = [
                child for child in self.children if not isinstance(child, Label)
            ]
            for child in children_to_remove:
                self.remove_widget(child)

        if self.current_mode in ["icon", "text"]:
            self.add_widget(Widget(), index=2)
            self.add_widget(Widget(), index=0)
        else:
            self.add_widget(Widget(), index=0)
            self.add_widget(Widget(), index=2)
            self.add_widget(Widget(), index=4)

    def on_kv_post(self, *args):
        if not self.text and not self.icon:
            self.text = "Button"


# fmt: off
Builder.load_string("""
<-CButton>:
    size_hint: None, None
    _width: None
    width: (label.texture_size[0] + icon.size[0] + dp(35) if icon.size[0] < dp(50) and icon.icon and label.text else label.texture_size[0] + icon.size[0]+ dp(20)) if not self._width else self._width

    _height: None
    height: (dp(50) if icon.size[1] < dp(50) else icon.size[1]) if not self._height else self._height

    canvas.before:
        Color:
            rgba: root.current_color
        RoundedRectangle:
            size: self.size
            pos: self.pos
            radius: root.radius
        Color:
            rgba: root.border_color if root.border_width else (0,0,0,0)
        SmoothLine:
            width: root.border_width
            rounded_rectangle: (self.x, self.y, self.width, self.height, root.radius[0])

    Label:
        id: icon
        text: u"{}".format(unicode[root.icon]) if root.icon in unicode else "blank"
        icon: root.icon
        font_name: icon_font
        size_hint: (None, None)
        size: self.texture_size if self.icon else (dp(0), dp(0))
        font_size: root.icon_size # this is the icon size
        pos_hint: {"center_x": .5, "center_y": .5}
        color: root.icon_color

        canvas.before:
            Color:
                rgba: root.current_color
            Rectangle:
                pos: self.pos
                size: self.size

    Label:
        id: label
        text: root.text
        font_size: root.font_size
        size_hint: None, None
        size: self.texture_size
        color: root.font_color
        pos_hint: {'center_y': .5}
""")
# fmt: on
